VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsFDatabase"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'     FDB
'  , 2006 - 2007

'   Huffman Compression Algorithm

Option Explicit

Private Const FDB_TEMP As String = "FDBDF.tmp"
Private Const FDB_MAX_FIELDS As Long = 127
Private Const FDB_SIGNATURE As String = "FDB"

Private Type FDBHEADER
    Signature As String * 8
    Version As String * 8
    Copyright As String * 128
    Comment As String * 128
End Type

Private Type FDBFIELDSINFO
    FieldName(FDB_MAX_FIELDS) As String * 64
    FieldOffset(FDB_MAX_FIELDS) As Long
    FieldLength(FDB_MAX_FIELDS) As Long
End Type

Private FDBFieldName As Collection
Private FDBFieldData As Collection

Private strFDBVersion As String
Private strFDBCopyright As String

Private HCA As New clsHuffman

Public Function OpenFDB(ByVal strFileName As String) As Boolean
    On Error Resume Next
    Call CheckFDB(strFileName)
    Dim strTF As String
    Dim f As Long
    Dim FDBH As FDBHEADER
    Dim FDBFI As FDBFIELDSINFO
    Dim strFName As String
    Dim strFData As String
    Dim i As Long
    Set FDBFieldName = New Collection
    Set FDBFieldData = New Collection
    strTF = IIf(Environ$("Temp") <> vbNullString, AddDirSep(Environ$("Temp")), _
        AddDirSep(App.Path)) & FDB_TEMP
    Call HCA.DecodeFile(strFileName, strTF)
    f = FreeFile
    Open strTF For Binary Access Read Lock Write As #f
        Get #f, , FDBH
        If UCase$(TrimEx(FDBH.Signature)) <> FDB_SIGNATURE Then Err.Raise 5
        strFDBVersion = TrimEx(FDBH.Version)
        strFDBCopyright = TrimEx(FDBH.Copyright)
        Get #f, , FDBFI
        For i = 0 To FDB_MAX_FIELDS
            strFName = FDBFI.FieldName(i)
            FDBFieldName.Add strFName, "key_" & i
            strFData = Space$(FDBFI.FieldLength(i))
            Get #f, FDBFI.FieldOffset(i), strFData
            FDBFieldData.Add strFData, "key_" & i
        Next i
    Close #f
    Kill strTF
    OpenFDB = CBool(Err = 0)
End Function

Public Function SaveFDB(ByVal strFileName As String, Optional ByVal strVersion As String = "1.1", _
    Optional ByVal strCopyright As String = "Copyright  Boris Vorontsov, 2006 - 2007", _
    Optional ByVal strComment As String = "FDatabase") As Boolean
    On Error Resume Next
    Dim strTF As String
    Dim f As Long
    Dim FDBH As FDBHEADER
    Dim FDBFI As FDBFIELDSINFO
    Dim strFName As String
    Dim strFData As String
    Dim lOffset As Long
    Dim i As Long
    strTF = IIf(Environ$("Temp") <> vbNullString, AddDirSep(Environ$("Temp")), _
        AddDirSep(App.Path)) & FDB_TEMP
    f = FreeFile
    Open strTF For Binary Access Write Lock Read As #f
        FDBH.Signature = FDB_SIGNATURE
        FDBH.Version = strVersion
        FDBH.Copyright = strCopyright
        FDBH.Comment = strComment
        Put #f, , FDBH
        lOffset = Len(FDBH) + Len(FDBFI)
        For i = 0 To FDB_MAX_FIELDS
            strFName = FDBFieldName.Item("key_" & i)
            FDBFI.FieldName(i) = strFName
            FDBFI.FieldLength(i) = Len(FDBFieldData.Item("key_" & i))
            FDBFI.FieldOffset(i) = lOffset
            lOffset = lOffset + FDBFI.FieldLength(i)
        Next i
        Put #f, , FDBFI
        For i = 0 To FDB_MAX_FIELDS
            strFData = FDBFieldData.Item("key_" & i)
            Put #f, FDBFI.FieldOffset(i), strFData
        Next i
    Close #f
    Call HCA.EncodeFile(strTF, strFileName)
    Kill strTF
    SaveFDB = CBool(Err = 0)
End Function

Public Function CloseFDB() As Boolean
    On Error Resume Next
    Set FDBFieldName = Nothing
    Set FDBFieldData = Nothing
    CloseFDB = CBool(Err = 0)
End Function

Private Sub CheckFDB(ByVal strFileName As String)
    Dim i As Long
    If Dir$(strFileName) = vbNullString Then
        Set FDBFieldName = New Collection
        Set FDBFieldData = New Collection
        For i = 0 To FDB_MAX_FIELDS
            FDBFieldName.Add String$(2, Chr$(0)), "key_" & i
            FDBFieldData.Add String$(2, Chr$(0)) & Space$(8), "key_" & i
        Next i
        Call SaveFDB(strFileName)
        Call CloseFDB
    End If
End Sub

Public Function WriteToFDB(ByRef lFieldIndex As Long, ByVal strName As String, ByVal strData As String) As Boolean
    On Error Resume Next
    If lFieldIndex = -1 Then
        Dim i As Long
        For i = 0 To FDB_MAX_FIELDS
            If Len(TrimEx(FDBFieldName.Item("key_" & i))) = 0 Then
                lFieldIndex = i
                Exit For
            End If
        Next i
    End If
    If lFieldIndex = -1 Then Err.Raise 5
    FDBFieldName.Remove "key_" & lFieldIndex
    FDBFieldData.Remove "key_" & lFieldIndex
    FDBFieldName.Add String$(2, Chr$(0)) & strName, "key_" & lFieldIndex
    FDBFieldData.Add String$(2, Chr$(0)) & strData, "key_" & lFieldIndex
    WriteToFDB = CBool(Err = 0)
End Function

Public Function ReadFromFDB(ByVal lFieldIndex As Long, ByRef strName As String, ByRef strData As String, _
    Optional ByVal bNameOnly As Boolean = False) As Boolean
    On Error Resume Next
    strName = TrimEx(FDBFieldName.Item("key_" & lFieldIndex))
    If strName = vbNullString Then Err.Raise 5
    If Not bNameOnly Then
        strData = TrimEx(FDBFieldData.Item("key_" & lFieldIndex))
    End If
    ReadFromFDB = CBool(Err = 0)
End Function

Public Function RemoveFromFDB(ByVal lFieldIndex As Long) As Boolean
    On Error Resume Next
    FDBFieldName.Remove "key_" & lFieldIndex
    FDBFieldData.Remove "key_" & lFieldIndex
    FDBFieldName.Add String$(2, Chr$(0)), "key_" & lFieldIndex
    FDBFieldData.Add String$(2, Chr$(0)) & Space$(8), "key_" & lFieldIndex
    RemoveFromFDB = CBool(Err = 0)
End Function

Public Property Get AvailableFields() As Long
    AvailableFields = FDB_MAX_FIELDS
End Property

Public Property Get FDBVersion() As String
    FDBVersion = strFDBVersion
End Property

Public Property Get FDBCopyright() As String
    FDBCopyright = strFDBCopyright
End Property

Private Function TrimEx(ByVal strString As String) As String
    If strString = vbNullString Then Exit Function
    strString = Replace(strString, Chr$(0), vbNullString)
    TrimEx = Trim$(strString)
End Function

Private Function AddDirSep(ByVal strPath As String) As String
    If strPath = vbNullString Then Exit Function
    If Right$(strPath, 1) <> Chr$(92) Then strPath = strPath & Chr$(92)
    AddDirSep = strPath
End Function
